Package: seededlda
Type: Package
Title: Seeded Sequential LDA for Topic Modeling
Version: 1.4.4
Authors@R: c(person("Kohei", "Watanabe", email = "watanabe.kohei@gmail.com", role = c("aut", "cre", "cph")),
             person("Phan", "Xuan-Hieu", email = "pxhieu@gmail.com", role = c("aut", "cph"), comment = "GibbsLDA++"))
Description: Seeded Sequential LDA can classify sentences of texts into pre-define topics with a small number of seed words (Watanabe & Baturo, 2023) <doi:10.1177/08944393231178605>.
    Implements Seeded LDA (Lu et al., 2010) <doi:10.1109/ICDMW.2011.125> and Sequential LDA (Du et al., 2012) <doi:10.1007/s10115-011-0425-1> with the distributed LDA algorithm (Newman, et al., 2009) for parallel computing.
License: GPL-3
LazyData: TRUE
URL: https://github.com/koheiw/seededlda,
        https://koheiw.github.io/seededlda/
BugReports: https://github.com/koheiw/seededlda/issues
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: quanteda (>= 4.0.0), methods, proxyC (>= 0.3.1), Matrix, utils
LinkingTo: Rcpp, RcppArmadillo (>= 0.7.600.1.0), quanteda, testthat
Suggests: spelling, testthat, topicmodels
RoxygenNote: 7.3.3
Language: en-US
NeedsCompilation: yes
Packaged: 2026-05-28 05:43:13 UTC; watan
Author: Kohei Watanabe [aut, cre, cph],
  Phan Xuan-Hieu [aut, cph] (GibbsLDA++)
Maintainer: Kohei Watanabe <watanabe.kohei@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-28 06:30:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-28 06:51:56 UTC; unix
Archs: seededlda.so.dSYM
