# Generated by roxygen2: do not edit by hand

S3method(as.dictionary,textmodel_lda)
S3method(divergence,textmodel_lda)
S3method(perplexity,textmodel_lda)
S3method(predict,textmodel_lda)
S3method(print,textmodel_lda)
S3method(sizes,textmodel_lda)
S3method(terms,textmodel_lda)
S3method(textmodel_lda,dfm)
S3method(textmodel_seededlda,dfm)
S3method(textmodel_seqlda,dfm)
S3method(topics,textmodel_lda)
export(divergence)
export(info_tbb)
export(perplexity)
export(sizes)
export(terms)
export(textmodel_lda)
export(textmodel_seededlda)
export(textmodel_seqlda)
export(topics)
import(quanteda)
importFrom(Matrix,Matrix)
importFrom(methods,as)
importFrom(proxyC,dist)
importFrom(utils,head)
useDynLib(seededlda, .registration = TRUE)
