#' Movie reviews from Pang and Lee (2004)
#'
#' A corpus object containing 2,000 movie reviews.
#' @name data_corpus_moviereviews
#' @references
#' Pang, B., Lee, L.  (2004)
#' "[A Sentimental Education: Sentiment Analysis Using Subjectivity Summarization Based on
#' Minimum Cuts.](https://www.cs.cornell.edu/home/llee/papers/cutsent.pdf)", Proceedings of the ACL.
#'
#' @source <https://www.cs.cornell.edu/people/pabo/movie-review-data/>
NULL
