% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{info_tbb}
\alias{info_tbb}
\title{Get information on TBB library}
\usage{
info_tbb()
}
\description{
Get information on TBB library
}
\keyword{internal}
